<?php
/**
 * This file implements the AstonishMe Allow Comments plugin.
 *
 * @package plugins
 *
 * {@internal Below is a list of authors who have contributed to design/coding of this file: }}
 * @author Yabba - {@link http://astonishme.co.uk/}
 *
 */
if( !defined('EVO_MAIN_INIT') ) die( 'Please, do not access this page directly.' );


/**
 * AM Allow Comments Plugin
 *
 * This plugin allows users to comment without moderation.
 *
 */
class am_allow_comments_plugin extends Plugin
{
	var $name = 'AM Allow Comments';
	var $code = 'am_allowcmnts';
	var $priority = 50;
	var $version = '1.0.1';
	var $author = 'AstonishMe';
	var $help_url = 'http://astonishme.co.uk/';

	function am_allow_comments_plugin()
	{
		$this->short_desc = $this->T_('AstonishMe Allow Comments');
		$this->long_desc = $this->T_('This plugin allows logged in users to bypass comment moderation.');
	}


	function GetDefaultSettings()
	{
		return array(
				'enabled' => array(
					'type' => 'checkbox',
					'label' => $this->T_('Enabled'),
					'note' => $this->T_('Untick this to disable this plugin'),
					'defaultvalue' => 1,
				),
				'visitor_url' => array(
					'type' => 'checkbox',
					'label' => $this->T_('Visitor website'),
					'note' => $this->T_('If ticked this will allow visitors to enter their website address whilst commenting'),
					'defaultvalue' => 1,
				),
				'visitor_karma' => array(
					'type' => 'integer',
					'size' => 4,
					'label' => $this->T_('Visitor website karma'),
					'note' => $this->T_('This is the spam karma that will be assigned if visitors leave a url when they\'re not allowed (values -100 > 100 )'),
					'defaultvalue' => 100,
					'valid_range' => array( 'min' => -100, 'max' => 100, 'error' => $this->T_( 'Please enter a value between -100 and 100' ) ) 
				),
				'visitor_comment_url' => array(
					'type' => 'checkbox',
					'label' => $this->T_('Visitors website(s) in comment content'),
					'note' => $this->T_('If ticked this will allow visitors to enter website address(s) in there comment content'),
					'defaultvalue' => 1,
				),
				'allow_author' => array(
					'type' => 'checkbox',
					'label' => $this->T_('Allow author comments'),
					'note' => $this->T_('If ticked this will allow the author of a post to bypass comment moderation'),
					'defaultvalue' => 1,
				),
				'allow_members' => array(
					'type' => 'checkbox',
					'label' => $this->T_('Allow members'),
					'note' => $this->T_('If ticked this will allow members of the posts blog to bypass comment moderation'),
					'defaultvalue' => 1,
				),
				'allow_users' => array(
					'type' => 'checkbox',
					'label' => $this->T_('Allow registered users'),
					'note' => $this->T_('If ticked this will allow ANY logged in user to bypass comment moderation (based on their user level)'),
					'defaultvalue' => 1,
				),
				'users_karma' => array(
					'type' => 'integer',
					'size' => 4,
					'label' => $this->T_('Users karma'),
					'note' => $this->T_('This is the spam karma multiplier that will be assigned if the comment is from a registered user (values -10 > 10 )'),
					'defaultvalue' => -10,
					'valid_range' => array( 'min' => -10, 'max' => 10, 'error' => $this->T_( 'Please enter a value between -10 and 10' ) ) 
				),
			);
	}

	function GetSpamKarmaForComment( & $params )
	{
		global $current_User;
		$karma = 0;
		
		if( $this->Settings->get('enabled') )
		{	// plugin is enabled, lets go to work
			$this->debug_log( $this->T_( 'plugin enabled' ) );
			if( isset( $current_User ) )
			{	// lets do the registered user checks
				$this->debug_log( $this->T_( 'current user set' ) );
				if( $this->Settings->get( 'allow_users' ) )
				{	// set karma based on user level
					$karma = $current_User->level * $this->Settings->get( 'users_karma' );
				$this->debug_log( $this->T_( 'user level karma set to : '.$karma ) );
				}
				if( $this->Settings->get( 'allow_members' ) and $current_User->check_perm( 'blog_ismember', 'any', false, $params[ 'Comment' ]->Item->Blog->ID ) )
				{	// set karma based on membership (overrides user level karma)
					$karma = -100;
				$this->debug_log( $this->T_( 'member level karma set to : '.$karma ) );
				}
				if( $this->Settings->get( 'allow_author' ) and ( in_array( $current_User->ID, array( $params[ 'Comment' ]->Item->creator_user_ID, $params[ 'Comment' ]->Item->lastedit_user_ID, $params[ 'Comment' ]->Item->assigned_user_ID ) ) ) )
				{	// set karma based on post author(s) (overrides user level karma and blog membership, although how they hell they post without being a blog member is beyond me ...... better safe than sorry huh? )
					$karma = -100;
					$this->debug_log( $this->T_( 'author level karma set to : '.$karma ) );
				}
			} elseif( !$this->Settings->get( 'visitor_url' ) and $params[ 'Comment' ]->author_url )
			{ // this is a visitor and they've wrongly sent us a url
				$karma = $this->Settings->get( 'visitor_karma' );
				$this->debug_log( $this->T_( 'visitor level karma set to : '.$karma ) );
			}
		};
		$this->debug_log( $this->T_( 'plugin karma set to : '.$karma ) );
		// need to ensure that the integer setting is actually returned as a number! :|
		return intval($karma);
	}

	function AfterCommentFormInsert( $params )
	{
		global $current_User;
		if( isset( $current_User )
				and $this->Settings->get('enabled')
				and ( 
					($this->Settings->get( 'allow_members' )
					and $current_User->check_perm( 'blog_ismember', 'any', false, $params[ 'Comment' ]->Item->Blog->ID ) )
					or( $this->Settings->get( 'allow_author' )
						and ( in_array( $current_User->ID, array( $params[ 'Comment' ]->Item->creator_user_ID, $params[ 'Comment' ]->Item->lastedit_user_ID, $params[ 'Comment' ]->Item->assigned_user_ID ) ) )
						)
					)
				)
		{	// this is an authorised user, publish the comment
			$params['Comment']->set( 'status', 'published');
			$params['Comment']->set_spam_karma( NULL );
			$params['Comment']->dbupdate();
			$this->debug_log( $this->T_( 'Authorised user. Comment published' ) );
		}
	}

	function SkinBeginHtmlHead()
	{
		if( !$this->Settings->get( 'visitor_url' ) )
		{	// visitors cannot leave a web address
			echo '<style type="text/css">
			#o{display:none;}
			</style>'."\n";
		}
	}
	
	function DisplayCommentFormButton()
	{
		if( !$this->Settings->get( 'visitor_url' ) )
		{	// visitors cannot leave a web address
			//if javascript is enabled then remove the url portion of the form completely
			echo '<script type="text/javascript">
				document.getElementById( "o" ).parentNode.parentNode.style.display = "none";
				</script>';
		}
	}

	function BeforeCommentFormInsert( $params )
	{
		if( $this->Settings->get('enabled') )
		{	// plugin is enabled, lets go to work
			if( !is_logged_in() and !$this->Settings->get( 'visitor_comment_url' ) )
			{	// lets do the url checks
				$text = $params['Comment']->content;
				if( strpos( $this->find_links( $text ), '[link]' ) !== false)
				{
					global $Messages;
					$Messages->add( $this->T_( 'Visitors are not allowed to leave links' ), 'error' );
				}
			}
		}
	}

	function find_links( $text, $moredelim = '&amp;' )
	{
		$text = callback_on_non_matching_blocks( $text, '~<a[^>]*("[^"]"|\'[^\']\')?[^>]*>.*?</a>~is', array( $this, 'find_links_callback'), array( $moredelim ) );
		$text = preg_replace( '~<a(.*?)href="(.+?)"(.*?)>(.*?)</a>~is', '[link]', $text );
		return $text;
	}


	function find_links_callback( & $text, $moredelim = '&amp;' )
	{
		$pattern_domain = '([a-z0-9\-]+\.[a-z0-9\-.\~]+)'; // a domain name (not very strict)
		$text = preg_replace(
			array( '#(^|[\s>])(https?|mailto)://([^<>{}\s]+[^.,<>{}\s])#i',
				'#(^|[\s>])www\.'.$pattern_domain.'((?:/[^<\s]*)?[^.,\s])#i',
				'#(^|[\s>])([a-z0-9\-_.]+?)@'.$pattern_domain.'([^.,<\s]+)#i', ),
			'[link]',
			$text );
		return $text;
	}

	function am_update( & $params ){ $params = array( 'name' =>$this->code, 'version' => $this->version ); }
}
?>